// Copyright (C) 2003 by Robert C. Martin and Micah D. Martin. All rights reserved.
// Released under the terms of the GNU General Public License version 2 or later.
package fitnesse2.http;

import fitnesse2.testutil.RegexTest;

public class ChunkedResponseTest extends RegexTest implements ResponseSender
{
	private ChunkedResponse response;
	private boolean closed = false;

	public StringBuffer buffer;

	public void send(byte[] bytes)
	{
		buffer.append(new String(bytes));
	}

	public void close()
	{
		closed = true;
	}

	public void setUp() throws Exception
	{
		buffer = new StringBuffer();

		response = new ChunkedResponse();
		response.readyToSend(this);
	}

	public void tearDown() throws Exception
	{
		response.done();
	}

	public void testHeaders() throws Exception
	{
		String text = buffer.toString();
		assertHasRegexp("Transfer-Encoding: chunked", text);
		assertTrue(text.startsWith("HTTP/1.1 200 OK\r\n"));
		assertHasRegexp("Content-Type: text/html", text);
	}

	public void testOneChunk() throws Exception
	{
		buffer = new StringBuffer();
		response.add("some more text");

		String text = buffer.toString();
		assertEquals("e\r\nsome more text\r\n", text);
	}

	public void testTwoChunks() throws Exception
	{
		buffer = new StringBuffer();
		response.add("one");
		response.add("two");

		String text = buffer.toString();
		assertEquals("3\r\none\r\n3\r\ntwo\r\n", text);
	}

	public void testSimpleClosing() throws Exception
	{
		assertFalse(closed);
		buffer = new StringBuffer();
		response.done();
		String text = buffer.toString();
		assertEquals("0\r\n\r\n", text);
		assertTrue(closed);
	}

	public void testContentSize() throws Exception
	{
		response.add("12345");
		response.done();
		assertEquals(5, response.getContentSize());
	}
}
