// Copyright (C) 2003 by Robert C. Martin and Micah D. Martin. All rights reserved.
// Released under the terms of the GNU General Public License version 2 or later.
package fitnesse2.http;

import fitnesse2.testutil.RegexTest;

public class SimpleResponseTest extends RegexTest implements ResponseSender
{
	private String text;
	private boolean closed = false;

	public void send(byte[] bytes)
	{
		text = new String(bytes);
	}

	public void close()
	{
		closed = true;
	}

	public void setUp() throws Exception
	{
	}

	public void tearDown() throws Exception
	{
	}

	public void testSimpleResponse() throws Exception
	{
		SimpleResponse response = new SimpleResponse();
		response.setContent("some content");
		response.readyToSend(this);
		assertTrue(text.startsWith("HTTP/1.1 200 OK\r\n"));
		assertHasRegexp("Content-Length: 12", text);
		assertHasRegexp("Content-Type: text/html", text);
		assertTrue(text.endsWith("some content"));
		assertTrue(closed);
	}

	public void testPageNotFound() throws Exception
	{
		SimpleResponse response = new SimpleResponse(404);
		response.readyToSend(this);
		assertHasRegexp("404 Not Found", text);
	}

	public void testRedirect() throws Exception
	{
		SimpleResponse response = new SimpleResponse();
		response.redirect("some url");
		response.readyToSend(this);
		assertEquals(303, response.getStatus());
		assertHasRegexp("Location: some url\r\n", text);
	}
}



