// Copyright (C) 2003 by Robert C. Martin and Micah D. Martin. All rights reserved.
// Released under the terms of the GNU General Public License version 2 or later.
package fitnesse2.socketservice;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.LinkedList;

public class SocketService
{
	private ServerSocket serverSocket = null;
	private Thread serviceThread = null;
	private boolean running = false;
	private SocketServer service = null;
	private LinkedList threads = new LinkedList();

	public SocketService(int port, SocketServer service) throws Exception
	{
		this.service = service;
		serverSocket = new ServerSocket(port);
		serviceThread = new Thread(
		   new Runnable()
		   {
			   public void run()
			   {
				   serviceThread();
			   }
		   }
		);
		serviceThread.start();
	}

	public void close() throws Exception
	{
		waitForServiceThreadToStart();
		running = false;
		serverSocket.close();
		serviceThread.join();
		waitForServerThreads();
	}

	private void waitForServiceThreadToStart()
	{
		while(running == false) Thread.yield();
	}

	private void serviceThread()
	{
		running = true;
		while(running)
		{
			try
			{
				Socket s = serverSocket.accept();
				startServerThread(s);
			}
			catch(IOException e)
			{
			}
		}
	}

	private void startServerThread(Socket s)
	{
		Thread serverThread = new Thread(new ServerRunner(s));
		synchronized(threads)
		{
			threads.add(serverThread);
		}
		serverThread.start();
	}

	private void waitForServerThreads() throws InterruptedException
	{
		while(threads.size() > 0)
		{
			Thread t;
			synchronized(threads)
			{
				t = (Thread) threads.getFirst();
			}
			t.join();
		}
	}

	private class ServerRunner implements Runnable
	{
		private Socket socket;

		ServerRunner(Socket s)
		{
			socket = s;
		}

		public void run()
		{
			try
			{
				service.serve(socket);
				synchronized(threads)
				{
					threads.remove(Thread.currentThread());
				}
				// Do we need to close the socket here?  FitNesse will already close all of the sockets. check the SDPS codebase
			}
			catch(Exception e)
			{
			}
		}
	}
}
