// Copyright (C) 2003 by Robert C. Martin and Micah D. Martin. All rights reserved.
// Released under the terms of the GNU General Public License version 2 or later.
package xpcinci;

import fitnesse2.*;
import fitnesse2.socketservice.SocketService;
import fitnesse2.responders.ResponderFactory;
import fitnesse2.components.Logger;
import com.neoworks.util.Getopts;

import java.net.BindException;
import java.io.File;

public class ECalendar
{	
	public static final String FITNESSE_VERSION = "20030728";
	public static final String ECAL_VERSION = "20030826";
	public static final String VERSION = FITNESSE_VERSION + ":" +ECAL_VERSION;

	public Logger logger = null;
	public String filesPath = "";
	private static SocketService theService;
	private int port = 80;

	public static void main(String[] args) throws Exception
	{
		Arguments arguments = parseCommandLine(args);
		if(arguments != null)
		{
			String filesPath = arguments.getRootPath();
			Logger logger = makeLogger(arguments);
			ECalendar fitnesse = new ECalendar(arguments.getPort(), filesPath, logger);
			boolean started = fitnesse.start();
			if(started)
				printStartMessage(arguments);
		}
		else
		{
			printUsage();
			System.exit(1);
		}
	}

	public static Arguments parseCommandLine(String[] args)
	{
		Arguments arguments = null;
		Getopts opts = new Getopts("p:d:l:", args);

		if(opts.error() == false)
		{
			arguments = new Arguments();
			arguments.setPort(opts.option('p', Arguments.DEFAULT_PORT));
			arguments.setRootPath(opts.option('d', Arguments.DEFAULT_DIRECTORY));
			arguments.setLogDirectory(opts.option('l', Arguments.DEFAULT_LOG_DIR));
		}
		return arguments;
	}

	private static Logger makeLogger(Arguments arguments)
	{
		String logDirectory = arguments.getLogDirectory();
		return logDirectory != null ? new Logger(logDirectory) : null;
	}


	private static void printUsage()
	{
		System.err.println("Usage: java fitnesse.Fitnesse [-pdl]");
		System.err.println("\t-p <port number> {" + Arguments.DEFAULT_PORT + "}");
		System.err.println("\t-d <working directory> {" + Arguments.DEFAULT_DIRECTORY + "}");
		System.err.println("\t-l <log directory> {no logging}");
	}

	private static void printStartMessage(Arguments args)
	{
		System.out.println("ECal Server (" + VERSION + ") Started...");
		System.out.println("\ton port      " + args.getPort());
		System.out.println("\tusing path   '" + args.getRootPath() + "'");
		if(args.getLogDirectory() == null)
			System.out.println("\tNot logging.");
		else
			System.out.println("\tlogging dir  '" + args.getLogDirectory() + "'");
	}

	private static void printBadPortMessage(int port)
	{
		System.err.println("ECal cannot be started...");
		System.err.println("Port " + port + " is already in use.");
		System.err.println("Use the -p <port#> command line argument to use a different port.");
	}
	
	private static void printShutdownMessage()
	{
		System.err.println("ECal shutting down...");
		System.err.println("+ + + + + + + + + + + + + + + + +");
	}

	private static void establishDirectory(String path)
	{
		File filesDir = new File(path);
		if(!filesDir.exists())
			filesDir.mkdir();
	}

	public ECalendar(int port, String filesPath, Logger logger) throws Exception
	{
		this(port, filesPath);
		this.logger = logger;
	}

	public ECalendar(int port, String filesPath) throws Exception
	{
		this.port = port;
		this.filesPath = filesPath;
		prepare();
	}

	public static void shutdown()
	{ 
		printShutdownMessage(); 
		System.exit(0);
	}

	public boolean start()
	{
		try
		{
			ResponderFactory responderFactory = new ResponderFactory(filesPath);
			theService = new SocketService(port, new FitnesseServer(responderFactory, logger));
			return true;
		}
		catch(BindException e)
		{
			printBadPortMessage(port);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		return false;
	}

	public void prepare() throws Exception
	{
		establishDirectory(filesPath);
		establishDirectory(filesPath + "/files");
	}


	public void stop() throws Exception
	{
		if(theService != null)
			theService.close();
	}
}
