package xpcinci.domain;

import java.io.*;
import java.text.*;
import java.util.*;

import xpcinci.domain.util.*;

public class Day implements Serializable {

	private int year, month, dayOfMonth;

	protected Day() {
	}

	public Day(String string) throws ParseException {
		Date date = (Date) DateParser.parse(string);
		loadDate(date);
	}

	public Day(Date date) {
		loadDate(date);
	}

	public Day(int y, int m, int d) {
		year = y;
		month = m;
		dayOfMonth = d;
	}

	public Day(int yyyymmdd) {
		dayOfMonth = yyyymmdd % 100;
		int yyyymm = yyyymmdd / 100;
		month = yyyymm % 100;
		int yyyy = yyyymm / 100;
		year = yyyy;
	}

	public int dayOfMonth() {
		return dayOfMonth;
	}

	private Date date() {
		return toDate(year, month, dayOfMonth);
	}

	private void loadDate(Date date) {
		GregorianCalendar cal = (GregorianCalendar) Calendar.getInstance();
		cal.setTime(date);
		year = cal.get(GregorianCalendar.YEAR);
		month = cal.get(GregorianCalendar.MONTH) + 1;
		dayOfMonth = cal.get(GregorianCalendar.DAY_OF_MONTH);
	}

	private Date toDate(int year, int month, int daynum) {
		GregorianCalendar cal = (GregorianCalendar) Calendar.getInstance();
		cal.set(GregorianCalendar.YEAR, year);
		cal.set(GregorianCalendar.MONTH, month - 1);
		cal.set(GregorianCalendar.DAY_OF_MONTH, daynum);
		return cal.getTime();
	}

	// --- factory
	private static DateFormat dateFormat = new SimpleDateFormat("ddMMMyyyy");

	public String toString() {
		return dateFormat.format(date());
	}

	//  --- util
	public boolean equals(Object obj) {

		if (!(obj instanceof Day))
			return false;
		Day other = (Day) obj;

		if (dayOfMonth != other.dayOfMonth)
			return false;
		if (month != other.month)
			return false;
		if (year != other.year)
			return false;
		return true;
	}

	public int hashCode() {
		return dayOfMonth + (10 * month);
	}

	public Day tommorow() {
		GregorianCalendar cal = calendar();
		cal.add(GregorianCalendar.DATE, 1);
		return new Day(cal.getTime());
	}

	public int dayOfWeek() {
		GregorianCalendar cal = calendar();
		return cal.get(Calendar.DAY_OF_WEEK);
	}

	private GregorianCalendar calendar() {
		GregorianCalendar cal = (GregorianCalendar) Calendar.getInstance();
		cal.setTime(date());
		return cal;
	}

	public int year() {
		return year;
	}

	public String weekOfYearAsString() {
		int weekNum = weekOfYear();
		String result = Integer.toString(weekNum);
		if (result.length() < 2)
			result = "0" + result;
		return result;
	}

	private int weekOfYear() {
		return calendar().get(Calendar.WEEK_OF_YEAR);
	}
}
