package xpcinci.domain;

import java.util.*;
import java.io.*;
import xpcinci.time.*;

public class Schedule implements Serializable {
	private static final String STREAM_DB_FILENAME = "Calendar.dat";

	ArrayList events = new ArrayList();
	LinkedList groups = new LinkedList();
	Week week01;

	private ScheduleElement[] prevQueryForTest = new ScheduleElement[0];
	public ScheduleElement[] prevQueryForTest() {
		return prevQueryForTest;
	}

	public Schedule(String curDate) {
		week01 = new Week(curDate);
	}

	final String defaultWeek = "200101";

	public Schedule() {
		week01 = new Week(defaultWeek);
	}

	private EventGroup existingOrNewGroup(String groupName) {
		EventGroup g = new EventGroup(groupName);
		if (groups.contains(g)) {
			g = (EventGroup) groups.get(groups.indexOf(g));
		} else {
			groups.add(g);
		}
		return g;
	}

	public String groupsTop() {
		return ((EventGroup) groups.getLast()).getName();
	}

	public int countOfEvents() {
		return events.size();
	}

	public int countOfGroups() {
		return groups.size();
	}

	public ScheduleElement[] allEvents() {
		return eventArray(events);
	}

	private ScheduleElement[] eventArray(List list) {
		ScheduleElement[] eventsFound =
			(ScheduleElement[]) list.toArray(new ScheduleElement[0]);
		return eventsFound;
	}

	public Week week01() {
		return week01;
	}
	public Week week02() {
		return week01.nextWeek();
	}
	public Week week03() {
		return week01.nextWeek().nextWeek();
	}
	public Week week04() {
		return week01.nextWeek().nextWeek().nextWeek();
	}

	public void addWeeks(int offset) {
		week01 = week01.plus(offset);
	}

	public void createEvent(
		String eventName,
		String groupName,
		TEScript when) {
		{
			EventGroup g = existingOrNewGroup(groupName);
			TimeExpression timeExpression = TEFactory.timeExpressionFrom(when);
			events.add(new ScheduleElement(eventName, g, timeExpression));
		}
	}

	public ScheduleElement[] eventsForWeek(Week week) {
		List result = new ArrayList();
		for (Iterator days = week.days(); days.hasNext();) {
			Day day = (Day) days.next();
			result.addAll(allEventsOccuring(day));
		}
		prevQueryForTest = new ScheduleElement[result.size()];
		return (ScheduleElement[]) result.toArray(prevQueryForTest);
	}

	// ==========================
	private List allEventsOccuring(Day day) {
		List result = new ArrayList();
		for (Iterator iter = events.iterator(); iter.hasNext();) {
			ScheduleElement element = (ScheduleElement) iter.next();
			if (element.occursOn(day))
				result.add(element);
		}
		return result;
	}

	public void save() throws IOException {
		ObjectOutputStream out =
			new ObjectOutputStream(new FileOutputStream(STREAM_DB_FILENAME));
		out.writeObject(events);
		out.writeObject(groups);
		out.close();
	}

	public void restore()
		throws FileNotFoundException, IOException, ClassNotFoundException {
		ObjectInputStream in =
			new ObjectInputStream(new FileInputStream(STREAM_DB_FILENAME));
		events = (ArrayList) in.readObject();
		groups = (LinkedList) in.readObject();
		in.close();
	}
}