package xpcinci.domain;

import java.util.Calendar;
import xpcinci.time.*;
import junit.framework.TestCase;

public class ScheduleTest extends TestCase {
	Schedule eCal;

	protected void setUp() throws Exception {
		super.setUp();
		eCal = new Schedule();
		eCal.createEvent("xpCinci", "programming", createDate(103, 0, 12));
	}

	private TEScript today() {
		Calendar calendar = Calendar.getInstance();

		int year = calendar.get(Calendar.YEAR);
		int month = calendar.get(Calendar.MONTH);
		int date = calendar.get(Calendar.DATE);
		return createDate(year, month, date);
	}

	private TEScript createDate(int year, int month, int date) {
		String result =
			"(time (year "
				+ (year - 100 + 2000)
				+ ")(month "
				+ (month + 1)
				+ ")(daynum "
				+ date
				+ "))";
		TEScript s = new TEScript(result);
		return s;
	}

	public void testCreateEvent() {
		assertEquals(1, eCal.countOfEvents());
	}

	public void testSaveRestore() throws Exception {
		assertEquals(1, eCal.countOfEvents());
		eCal.save();
		eCal = new Schedule();
		assertEquals(0, eCal.countOfEvents());
		eCal.restore();
		assertEquals(1, eCal.countOfEvents());
	}

	public void testSameGroup() {
		eCal.createEvent("cinjug", "programming", today());
		assertEquals(2, eCal.countOfEvents());
		assertEquals(1, eCal.countOfGroups());
	}

	public void testDiffGroup() {
		eCal.createEvent("hornTrio", "music", today());
		assertEquals(2, eCal.countOfEvents());
		assertEquals(2, eCal.countOfGroups());
	}

	public void testQuery() {
		eCal.createEvent("hornTrio", "music", today());
		assertEquals(2, eCal.countOfEvents());
		assertEquals(2, eCal.countOfGroups());
		ScheduleElement[] all = eCal.allEvents();
		assertEquals(2, all.length);
		assertEquals("xpCinci", all[0].getName());
		assertEquals("hornTrio", all[1].getName());

	}

	public void testEventsForWeek() {
		Week week = new Week(200301);
		ScheduleElement[] actual = eCal.eventsForWeek(week);
		assertEquals(0,actual.length); 
		
		Week week3 = new Week(200303);
		actual = eCal.eventsForWeek(week3);
		assertEquals(1,actual.length); 
		assertEquals("xpCinci",actual[0].getName()); 
	}

	public ScheduleTest(String name) {
		super(name);
	}
	
	
}
