package xpcinci.domain;

import java.util.*;

import junit.framework.*;

public class WeekTest extends TestCase {
	public WeekTest(String arg0) {
		super(arg0);
	}

	public static void main(String[] args) {
		junit.swingui.TestRunner.run(WeekTest.class);
	}

	public void testGetQueryDates() {
		Week w = new Week(200401);
		Iterator actual = w.days();
		Day sun = (Day) actual.next();
		assertEquals("28Dec2003", sun.toString());

		Day mon = (Day) actual.next();
		assertEquals("29Dec2003", mon.toString());

		Day tue = (Day) actual.next();
		assertEquals("30Dec2003", tue.toString());

		Day wen = (Day) actual.next();
		assertEquals("31Dec2003", wen.toString());

		Day thu = (Day) actual.next();
		assertEquals("01Jan2004", thu.toString());

		Day fri = (Day) actual.next();
		assertEquals("02Jan2004", fri.toString());

		Day sat = (Day) actual.next();
		assertEquals("03Jan2004", sat.toString());

	}

	public void testGetMonth1() {
		assertEquals(11, new Week("200301").getMonthOfSunday());
	}
	public void testGetMonth2() {
		assertEquals(0, new Week("200302").getMonthOfSunday());
	}
	public void testGetMonth3() {
		assertEquals(1, new Week("200306").getMonthOfSunday());
	}
	public void testGetMonth4() {
		assertEquals(11, new Week("200352").getMonthOfSunday());
	}

	public void testGetDayNumbers() throws Exception {
		Week week = new Week("200302");
		int[] expectedNumbers = { 5, 6, 7, 8, 9, 10, 11 };

		int[] dayNumbers = week.getDayNumbers();

		for (int i = 0; i < dayNumbers.length; i++)
			assertEquals(expectedNumbers[i], dayNumbers[i]);
	}

	public void testGetDayNumbersMonthBoundry() throws Exception {
		Week week = new Week("200301");
		int[] expectedNumbers = { 29, 30, 31, 1, 2, 3, 4 };

		int[] dayNumbers = week.getDayNumbers();

		for (int i = 0; i < dayNumbers.length; i++)
			assertEquals(expectedNumbers[i], dayNumbers[i]);
	}

	public void testGetMonthOfSaturday() {
		Week w = new Week("200403");
		assertEquals(0, w.getMonthOfSaturday());
	}

	public void testCreate() {
		Week w = new Week("2001,04");
		assertEquals(200104, w.value());
	}

	public void testCreateRoll() {
		Week w = new Week("2003,54");
		assertEquals(200401, w.value());
	}

	public void testCreateTooBig() {
		try {
			Week w = new Week("2003,55");
			fail("should throw Exception");
		} catch (Exception x) {
			assertEquals("week value too big: 55",x.getMessage());
		}
	}
	public void testCreate2() {
		Week w = new Week("2002,12");
		assertEquals(200212, w.value());
	}

	public void testRemoveComma() {
		Week w = new Week("2000,02");
		assertEquals("199912", w.removeCommas("1999,12"));
	}

	public void testToString() {
		Week w = new Week("2002,12");
		assertEquals("200212", w.toString());
	}

	public void testGetYear() {
		Week w = new Week("2003,03");
		assertEquals("2003", w.getYear());
	}

	public void testGetWeek() {
		Week w = new Week("2003,03");
		assertEquals("03", w.getWeek());
	}

	public void testNextWeek() {
		Week w = new Week("2003,03");
		Week expected = new Week("2003,04");
		assertEquals("200304", w.nextWeek().toString());
		assertEquals(expected.toString(), w.nextWeek().toString());
	}

	public void testNotEqualType() {
		Week one = new Week("2003,03");
		assertFalse(one.equals(new Integer(2)));
	}

	public void testEqualOther() {
		Week one = new Week("2003,03");
		Week two = new Week("2003,03");
		assertTrue(one.equals(two));
	}

	public void testEqualString() {
		Week one = new Week("2002,03");
		assertTrue(one.equals("2002,03"));
		Week two = new Week("2002,05");
		assertTrue(two.equals("200205"));
	}

	public void testEqualNum() {
		Week one = new Week("2001,03");
		assertTrue(one.equals(200103));
	}

	public void testRemoveDay() {
		Week one = new Week(20010301);
		assertTrue(one.equals(200103));
	}

	public void testInvalidBig() {
		try {
			new Week(200103014);
		} catch (RuntimeException x) {
			return;
		}
		fail("should throw exception");
	}

	public void testAddWeek() {
		Week week = new Week("2001,01");
		Week after = week.plus(1);
		assertEquals(new Week("200102"), after);
	}

	public void testAdd4Week() {
		Week week = new Week("2001,01");
		Week after = week.plus(4);
		assertEquals(new Week("200105"), after);
	}

	public void testAdd4WeekNewYear() {
		Week week = new Week("2000,52");
		Week after = week.plus(4);
		assertEquals(new Week("200103"), after);
	}

	public void testAdd1WeekNewYear_2() {
		Week week = new Week("2000,53");
		Week after = week.plus(1);
		assertEquals(new Week("200101"), after);
	}

	public void testAdd4WeekNewYear_2() {
		Week week = new Week("2000,50");
		Week after = week.plus(4);
		assertEquals(new Week("200101"), after);
	}

	public void testAdd4WeekNewYear_3() {
		Week week = new Week("2000,51");
		Week after = week.plus(4);
		assertEquals(new Week("200102"), after);
	}

	public void testAdd4WeekNewYear_4() {
		Week week = new Week("2000,49");
		Week after = week.plus(4);
		assertEquals(new Week("200053"), after);
	}

	public void testMinus4WeekNewYear_2() {
		Week week = new Week("2001,01");
		Week after = week.plus(-4);
		assertEquals(new Week("200050"), after);
	}

	public void testMinus1Week() {
		Week week = new Week("2001,01");
		assertEquals(new Week("200101"), week);
		Week after = week.plus(-1);
		assertEquals(new Week("200053"), after);
	}

	public void testMinus4Week() {
		Week week = new Week("2001,01");
		assertEquals(new Week("200101"), week);
		Week after = week.plus(-4);
		assertEquals(new Week("200050"), after);
	}
}
