package xpcinci.render;

import java.util.*;

import junit.framework.*;
import xpcinci.domain.*;
import xpcinci.domain.Schedule;
import xpcinci.responders.*;

public class RenderCalendarTest extends TestCase {

	public RenderCalendarTest(String arg0) {
		super(arg0);
	}

	public void testRenderSplitMonth() {
		RenderCalendar render = new RenderCalendar();
		render.current = new Schedule("200401");
		String title = render.titleMonthName();
		assertEquals("December/January", title);
	}

	public void testRenderDualMonth() {
		RenderCalendar render = new RenderCalendar();
		render.current = new Schedule("200402");
		assertEquals("200402", "January", render.titleMonthName());

		render.current = new Schedule("200406");
		assertEquals("200406", "February", render.titleMonthName());

		render.current = new Schedule("200407");
		assertEquals("200407", "February/March", render.titleMonthName());
	}

	public void testName() {
		RenderCalendar render = new RenderCalendar();
		render.displayLine = 1;
		assertEquals("name=\"cell_14\"", render.name(4));
	}

	public void testHref() {
		RenderCalendar render = new RenderCalendar();
		render.renderWeek = new Week(200401);
		assertEquals("href=\"ECal?cur=20040105&cmd=day\" ", 
					render.href(new Day(20040101)));
	}

	String callPath = "";
	class RenderCalendarForUsing extends RenderCalendar {
		void renderWeek(Week week) {
			callPath += "renderWeek:";
		}
		String titleMonthName() {
			return "titleMonthName";
		}
	}

	class MockSchedule extends Schedule {
		public Week week01() {
			callPath += "week01:";
			return null;
		}
		public Week week02() {
			callPath += "week02:";
			return null;
		}
		public Week week03() {
			callPath += "week03:";
			return null;
		}
		public Week week04() {
			callPath += "week04:";
			return null;
		}
	}

	public void testUsing() {
		RenderCalendar rc = new RenderCalendarForUsing();
		MockSchedule mock = new MockSchedule();
		Map map = rc.using(mock);
		assertEquals(
			"week01:renderWeek:week02:renderWeek:week03:renderWeek:week04:renderWeek:",
			callPath);
		assertEquals("", map.get(RESULTS.CALENDAR));
		assertEquals("titleMonthName", map.get(RESULTS.MONTHNAME));
	}

	class RenderCalendarForRenderDay extends RenderCalendar {
		String href(Day day) {
			return "href"+day.dayOfWeek();
		}
		String name(int dayOfWeek) {
			return "name"+dayOfWeek;
		}
	}
	public void testRenderDay() {
		RenderCalendar rc = new RenderCalendarForRenderDay();
		Day day = new Day(2004,1,1);
		rc.renderDay(day);
		assertEquals("<td><a href5name5>1</a></td>", rc.cal.toString());
	}

	class RenderCalendarForRenderWeek extends RenderCalendar {
		void renderDay( Day aday) {
			cal.append("renderDay(" + aday.dayOfMonth() +")");
		}
	}
	public void testRenderWeek() {
		RenderCalendar rc = new RenderCalendarForRenderWeek();
		rc.renderWeek(new Week(200331));
		assertEquals(
			"<tr>renderDay(27)renderDay(28)renderDay(29)"+
				"renderDay(30)renderDay(31)renderDay(1)renderDay(2)</tr>\n",
			rc.cal.toString());
	}

}