package xpcinci.template;

import java.io.*;
import java.util.*;

public class FastToken {
	int start;
	int end;
	final String raw;
	final PrintWriter pw;
	final Map keys;

	public FastToken(String string, PrintWriter pw, Map keys) {
		this.raw = string;
		this.pw = pw;
		this.keys = keys;
		fastFilter();
	}

	private void fastFilter() {
		start = raw.indexOf("<%");
		end = raw.indexOf("%>", start);
		if (start < 0 || end < 0) {
			pw.print(raw); // if broken or no token dump the rest.
			return;
		}

		//print before
		pw.print(raw.substring(0, start));

		final String key = raw.substring(start + 2, end).trim();
		final Object value = keys.get(key);
		if (value == null)
			unknownSubsitution(key);
		else {
			knownSubstitution(value);
		}
		final String after = raw.substring(end + 2, raw.length()).trim();
		if (after.length() > 0) {
			new FastToken(after, pw, keys);
		}
	}

	private void knownSubstitution(final Object value) {
		if (value instanceof String)
			pw.print((String) value);
		else
			unexpectedType(value);
	}

	private void unexpectedType(final Object value) {
		throw new RuntimeException(
			"Expecting String, found: " + value.getClass().getName());
	}

	private void unknownSubsitution(final String key) {
		pw.print("<%" + key + "%>");
	}
}
