package xpcinci.time;
import java.util.*;

public class TEScript  {

	final static List keywords = Arrays.asList(TE.keywords_array);
	final static List constants = Arrays.asList(TE.constants_array);

	String content;
	String keyword;
	String constant;
	TEScript child;
	TEScript peer;

	public String toString() {
		String result = "(";
		result += keyword + " ";
		if (child != null)
			result += child.toString();
		else
			result += constant;

		result += ")";
		if (peer != null)
			result += peer.toString();

		return result;
	}

	public TEScript(String content) {
		this.content = content;
		parseIn();
	}
	
	public TEScript parse() {return this;}

	public TEScript parseIn() {

		removeOpenParen();
		removeKeyword();

		if (hasChild()) {
			parseChild();
			return this;
		}
		removeConstant();
		removeCloseParen();
		parsePeer();
		return this;
	}

	private void parseChild() {
		int open = content.indexOf('(');
		if (open < 0)
			return;
		child = new TEScript(content.substring(open));
	}

	private void parsePeer() {
		int open = content.indexOf('(');
		if (open < 0)
			return;
		TEScript script = new TEScript(content.substring(open));
		peer = script;
	}

	private void removeKeyword() {
		content = content.trim();
		int space = indexOf(' ');
		int open = indexOf('(');
		if(ignoreExtraParens(open))
			return;
		
		assertAtLeastOneFound(space, open);
		int keyClose = (open < space) ? open : space;
		keyword = content.substring(0, keyClose);
		isKeyword(keyword);
		content = content.substring(keyClose);
	}

	private int indexOf(char delimiter) {
		int found = content.indexOf(delimiter);
		found = (found == -1) ? Integer.MAX_VALUE : found;
		return found;
	}

	private void assertAtLeastOneFound(int space, int open) {
		if (open == Integer.MAX_VALUE && open == space) 
			throw new TEScriptException("Missing Keyword: " + content);
	}

	private boolean ignoreExtraParens(int open) {
		if (open ==0)
		{	removeOpenParen();
			removeKeyword();
			return true;
		}
		return false;
	}

	private void removeCloseParen() {
		int close = content.indexOf(")");
		if (close < 0)
			throw new TEScriptException("Missing Close Paren: " + content);
		content = content.substring(close);
	}

	private void removeOpenParen() {
		int open = content.indexOf("(");
		if (open < 0)
			throw new TEScriptException("Missing Open Paren");
		content = content.substring(open + 1);
	}

	boolean hasChild() {
		int open = content.indexOf('(');
		int close = content.indexOf(')');
		if (open == -1)
			open = Integer.MAX_VALUE;
		if (close == -1)
			close = Integer.MAX_VALUE;
		return open < close;
	}

	private void isKeyword(String key) {
		if (!keywords.contains(key.toLowerCase().trim()))
			throw new TEScriptException("Invalid Keyword: [" + key + "]");
	}

	private void removeConstant() {
		int close = content.indexOf(')');
		if (close == -1) {
			constant = null;
			return;
		}
		constant = content.substring(0, close).trim();
		if (constants.contains(constant.toLowerCase())) {
			content = content.substring(close);
			return;
		}
		try {
			new Integer(constant);
			content = content.substring(close);
			return;
		} catch (NumberFormatException e) {
		}
		throw new TEScriptException("Invalid constant [" + constant + "]");
	}

	public int getInt(String find) {
		isKeyword(find);
		if (find.equalsIgnoreCase(keyword))
			return Integer.parseInt(constant);
		if (peer == null)
			throw new RuntimeException("Cannot find keyword [" + find + "]");
		return peer.getInt(find);
	}

	private void peers(List collectingParameter) {
		collectingParameter.add(keyword);
		if (peer == null)
			return;
		peer.peers(collectingParameter);
	}

	public List contentNames() {
		List list = new ArrayList();
		if (child ==null)
			return list;
		child.peers(list);
		return list;
	}
}
